@@ -30,6 +30,13 @@
 GST_DEBUG_CATEGORY_STATIC (gsmenc_debug);
 #define GST_CAT_DEFAULT (gsmenc_debug)
 
+/* elementfactory information */
+static const GstElementDetails gst_gsmenc_details =
+GST_ELEMENT_DETAILS ("GSM audio encoder",
+    "Codec/Encoder/Audio",
+    "Encodes GSM audio",
+    "Philippe Khalaf <burger@speedy.org>");
+
 /* GSMEnc signals and args */
 enum
 {
@@ -100,21 +107,21 @@ gst_gsmenc_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gsmenc_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gsmenc_src_template);
-  gst_element_class_set_details_simple (element_class, "GSM audio encoder",
-      "Codec/Encoder/Audio",
-      "Encodes GSM audio", "Philippe Khalaf <burger@speedy.org>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gsmenc_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gsmenc_src_template));
+  gst_element_class_set_details (element_class, &gst_gsmenc_details);
 }
 
 static void
 gst_gsmenc_class_init (GstGSMEnc * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
 
   gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
