@@ -81,7 +81,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include "config.h"
 #endif
 
 #include <string.h>
@@ -90,12 +90,23 @@
 #include <gst/rtp/gstrtpbuffer.h>
 #include "gstrtpdtmfdepay.h"
 
+#ifndef M_PI
+# define M_PI           3.14159265358979323846  /* pi */
+#endif
+
+
 #define DEFAULT_PACKET_INTERVAL  50     /* ms */
 #define MIN_PACKET_INTERVAL      10     /* ms */
 #define MAX_PACKET_INTERVAL      50     /* ms */
 #define SAMPLE_RATE              8000
 #define SAMPLE_SIZE              16
 #define CHANNELS                 1
+#define MIN_EVENT                0
+#define MAX_EVENT                16
+#define MIN_VOLUME               0
+#define MAX_VOLUME               36
+#define MIN_INTER_DIGIT_INTERVAL 100
+#define MIN_PULSE_DURATION       250
 #define MIN_DUTY_CYCLE           (MIN_INTER_DIGIT_INTERVAL + MIN_PULSE_DURATION)
 
 #define MIN_UNIT_TIME            0
@@ -106,7 +117,7 @@
 
 typedef struct st_dtmf_key
 {
-  const char *event_name;
+  char *event_name;
   int event_encoding;
   float low_frequency;
   float high_frequency;
@@ -153,6 +164,13 @@ enum
   DTMF_KEY_EVENT_D = 15,
 };
 
+/* elementfactory information */
+static const GstElementDetails gst_rtp_dtmfdepay_details =
+GST_ELEMENT_DETAILS ("RTP DTMF packet depayloader",
+    "Codec/Depayloader/Network",
+    "Generates DTMF Sound from telephone-event RTP packets",
+    "Youness Alaoui <youness.alaoui@collabora.co.uk>");
+
 GST_DEBUG_CATEGORY_STATIC (gst_rtp_dtmf_depay_debug);
 #define GST_CAT_DEFAULT gst_rtp_dtmf_depay_debug
 
@@ -216,27 +234,26 @@ gst_rtp_dtmf_depay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_dtmf_depay_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_dtmf_depay_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_dtmf_depay_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_dtmf_depay_sink_template));
 
 
   GST_DEBUG_CATEGORY_INIT (gst_rtp_dtmf_depay_debug,
       "rtpdtmfdepay", 0, "rtpdtmfdepay element");
-  gst_element_class_set_details_simple (element_class,
-      "RTP DTMF packet depayloader", "Codec/Depayloader/Network",
-      "Generates DTMF Sound from telephone-event RTP packets",
-      "Youness Alaoui <youness.alaoui@collabora.co.uk>");
+  gst_element_class_set_details (element_class, &gst_rtp_dtmfdepay_details);
 }
 
 static void
 gst_rtp_dtmf_depay_class_init (GstRtpDTMFDepayClass * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
   GstBaseRTPDepayloadClass *gstbasertpdepayload_class;
 
   gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
@@ -249,14 +266,13 @@ gst_rtp_dtmf_depay_class_init (GstRtpDTMFDepayClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_UNIT_TIME,
       g_param_spec_uint ("unit-time", "Duration unittime",
           "The smallest unit (ms) the duration must be a multiple of (0 disables it)",
-          MIN_UNIT_TIME, MAX_UNIT_TIME, DEFAULT_UNIT_TIME,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          MIN_UNIT_TIME, MAX_UNIT_TIME, DEFAULT_UNIT_TIME, G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_MAX_DURATION,
       g_param_spec_uint ("max-duration", "Maximum duration",
           "The maxumimum duration (ms) of the outgoing soundpacket. "
           "(0 = no limit)", 0, G_MAXUINT, DEFAULT_MAX_DURATION,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 
   gstbasertpdepayload_class->process =
       GST_DEBUG_FUNCPTR (gst_rtp_dtmf_depay_process);
@@ -496,7 +512,7 @@ gst_rtp_dtmf_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   }
 
   GST_DEBUG_OBJECT (depayload, "new previous duration : %d - new duration : %d"
-      " - diff  : %d - clock rate : %d - timestamp : %" G_GUINT64_FORMAT,
+      " - diff  : %d - clock rate : %d - timestamp : %llu",
       rtpdtmfdepay->previous_duration, dtmf_payload.duration,
       (rtpdtmfdepay->previous_duration - dtmf_payload.duration),
       depayload->clock_rate, GST_BUFFER_TIMESTAMP (buf));
@@ -516,8 +532,7 @@ gst_rtp_dtmf_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     GST_BUFFER_OFFSET_END (outbuf) = rtpdtmfdepay->previous_duration *
         GST_SECOND / depayload->clock_rate;
 
-    GST_DEBUG_OBJECT (depayload,
-        "timestamp : %" G_GUINT64_FORMAT " - time %" GST_TIME_FORMAT,
+    GST_DEBUG_OBJECT (depayload, "timestamp : %llu - time %" GST_TIME_FORMAT,
         GST_BUFFER_TIMESTAMP (buf), GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
 
   }
